package com.mortals.xhx.feign.mid;

import com.mortals.framework.common.Rest;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.mid.MidReq;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

@FeignClient(name = "base-manager", path = "/base", fallbackFactory = MidFeignFallbackFactory.class)
public interface IMidFeign {

    /**
     * 微中台透传接口
     * @param midReq
     * @return
     */
    @PostMapping(value = "/mid/trans")
    Rest<String> trans(@RequestBody MidReq midReq);
}

@Slf4j
@Component
class MidFeignFallbackFactory implements FallbackFactory<IMidFeign> {

    @Override
    public IMidFeign create(Throwable cause) {
        return new  IMidFeign(){

            @Override
            public Rest<String> trans(MidReq midReq) {
                return Rest.fail("暂时无法获取微中台透传接口，请稍后再试!");
            }
        };
    }
}
