package com.mortals.xhx.common.utils;

import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.checkerframework.checker.units.qual.C;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


public class ExcelUtil {

    /**
     * @Title: createExcelTemplate
     * @Description: 生成Excel导入模板
     * @param filePath  Excel文件路径
     * @param title   Excel列标题(数组)
     * @param downData  下拉框数据(数组)
     * @param downRows  下拉列的序号(数组,序号从0开始)
     * @return void
     * @throws
     */
    public static void createExcelTemplate(String filePath, String[] title, List<String[]> downData, String[] downRows){
        HSSFWorkbook wb = new HSSFWorkbook();//创建工作薄
        //表头样式
        HSSFCellStyle style = wb.createCellStyle();
        // 创建一个居中格式
        // style.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        //字体样式
        HSSFFont fontStyle = wb.createFont();
        fontStyle.setFontName("微软雅黑");
        fontStyle.setFontHeightInPoints((short)12);
        // fontStyle.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        style.setFont(fontStyle);

        // 新建sheet
        HSSFSheet sheet1 = wb.createSheet("Sheet1");
//        HSSFSheet sheet2 = wb.createSheet("Sheet2");
//        HSSFSheet sheet3 = wb.createSheet("Sheet3");

        // 生成sheet1内容
        // 第一个sheet的第一行为标题
        HSSFRow rowFirst = sheet1.createRow(0);
        // 写标题
        for(int i=0;i<title.length;i++){
            // 获取第一行的每个单元格
            HSSFCell cell = rowFirst.createCell(i);
            // 设置每列的列宽
            sheet1.setColumnWidth(i, 4000);
            //加样式
            cell.setCellStyle(style);
            // 往单元格里写数据
            cell.setCellValue(title[i]);
        }

        // 设置下拉框数据
        String[] arr = {"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z"};
        int index = 0;
        HSSFRow row = null;
        for(int r=0;r<downRows.length;r++){
            // 获取下拉对象
            String[] dlData = downData.get(r);
            int rownum = Integer.parseInt(downRows[r]);
            // 255以内的下拉
            if(dlData.length<5){
                // 255以内的下拉,参数分别是：作用的sheet、下拉内容数组、起始行、终止行、起始列、终止列
                sheet1.addValidationData(setDataValidation(sheet1, dlData, 1, 500, rownum ,rownum)); //超过255个报错
            } else {
                // 255以上的下拉，即下拉列表元素很多的情况
                // 1、设置有效性
                // Sheet2第A1到A5000作为下拉列表来源数据
                // String strFormula = "Sheet2!$A$1:$A$5000" ;
                // Sheet2第A1到A5000作为下拉列表来源数据
                String strFormula = "Sheet2!$"+arr[index]+"$1:$"+arr[index]+"$5000";
                // 设置每列的列宽
                //sheet2.setColumnWidth(r, 4000);
                // 设置数据有效性加载在哪个单元格上,参数分别是：从sheet2获取A1到A5000作为一个下拉的数据、起始行、终止行、起始列、终止列
                //下拉列表元素很多的情况
                sheet1.addValidationData(SetDataValidation(strFormula, 1, 50000, rownum, rownum));

                //2、生成sheet2内容
//                for(int j=0;j<dlData.length;j++){
//                    if(index==0){ //第1个下拉选项，直接创建行、列
//                        // 创建数据行
//                        row = sheet2.createRow(j);
//                        // 设置每列的列宽
//                        sheet2.setColumnWidth(j, 4000);
//                        // 设置对应单元格的值
//                        row.createCell(0).setCellValue(dlData[j]);
//
//                    } else { //非第1个下拉选项
//
//                        int rowCount = sheet2.getLastRowNum();
//                        //System.out.println("========== LastRowNum =========" + rowCount);
//                        // 前面创建过的行，直接获取行，创建列
//                        if(j<=rowCount){
//                            // 获取行，创建列
//                            // 设置对应单元格的值
//                            sheet2.getRow(j).createCell(index).setCellValue(dlData[j]);
//
//                        } else { //未创建过的行，直接创建行、创建列
//                            // 设置每列的列宽
//                            sheet2.setColumnWidth(j, 4000);
//                            // 创建行、创建列
//                            // 设置对应单元格的值
//                            sheet2.createRow(j).createCell(index).setCellValue(dlData[j]);
//                        }
//                    }
//                }
                index++;
            }
        }

        try {

            File f = new File(filePath); //写文件

            //不存在则新增
            if(!f.getParentFile().exists()){
                f.getParentFile().mkdirs();
            }
            if(!f.exists()){
                f.createNewFile();
            }

            FileOutputStream out = new FileOutputStream(f);
            out.flush();
            wb.write(out);
            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     *
     * @Title: SetDataValidation
     * @Description: 下拉列表元素很多的情况 (255以上的下拉)
     * @param strFormula
     * @param firstRow   起始行
     * @param endRow     终止行
     * @param firstCol   起始列
     * @param endCol     终止列
     * @return
     * @return HSSFDataValidation
     * @throws
     */
    private static HSSFDataValidation SetDataValidation(String strFormula, int firstRow, int endRow, int firstCol, int endCol) {
        // 设置数据有效性加载在哪个单元格上。四个参数分别是：起始行、终止行、起始列、终止列
        CellRangeAddressList regions = new CellRangeAddressList(firstRow, endRow, firstCol, endCol);
        DVConstraint constraint = DVConstraint.createFormulaListConstraint(strFormula);
        HSSFDataValidation dataValidation = new HSSFDataValidation(regions,constraint);

        dataValidation.createErrorBox("Error", "Error");
        dataValidation.createPromptBox("", null);
        return dataValidation;
    }

    /**
     *
     * @Title: setDataValidation
     * @Description: 下拉列表元素不多的情况(255以内的下拉)
     * @param sheet
     * @param textList
     * @param firstRow
     * @param endRow
     * @param firstCol
     * @param endCol
     * @return DataValidation
     * @throws
     */
    private static DataValidation setDataValidation(Sheet sheet, String[] textList, int firstRow, int endRow, int firstCol, int endCol) {

        DataValidationHelper helper = sheet.getDataValidationHelper();
        //加载下拉列表内容
        DataValidationConstraint constraint = helper.createExplicitListConstraint(textList);
        //DVConstraint constraint = new DVConstraint();
        constraint.setExplicitListValues(textList);

        //设置数据有效性加载在哪个单元格上。四个参数分别是：起始行、终止行、起始列、终止列
        CellRangeAddressList regions = new CellRangeAddressList(firstRow, endRow,firstCol, endCol);
        //数据有效性对象
        DataValidation data_validation = helper.createValidation(constraint, regions);
        //DataValidation data_validation = new DataValidation(regions, constraint);
        return data_validation;
    }

    /**
     * @Title: delFile
     * @Description: 删除文件
     * @param filePath  文件路径
     * @return void
     * @throws
     */
    public static void delFile(String filePath) {
        java.io.File delFile = new java.io.File(filePath);
        delFile.delete();
    }

    public static void createCatalogTemplate(String filePath,List<String> formInfo){
        List<String> basisInfo = new ArrayList<>();
        basisInfo.add("证照编号");
        basisInfo.add("证照名称");
        basisInfo.add("持有者名称");
        basisInfo.add("持有者类型");
        basisInfo.add("持有者证件类型");
        basisInfo.add("持有者证件号码");
        basisInfo.add("取件人姓名");
        basisInfo.add("取件人证件号码");
        basisInfo.add("颁发时间");
        basisInfo.add("专网ID");
        basisInfo.add("企业名称");
        HSSFWorkbook wb = new HSSFWorkbook();//创建工作薄
        //表头样式
        HSSFCellStyle style = wb.createCellStyle();
        // 创建一个居中格式
        style.setAlignment(HorizontalAlignment.CENTER);
        //字体样式
        HSSFFont fontStyle = wb.createFont();
        fontStyle.setFontName("微软雅黑");
        fontStyle.setFontHeightInPoints((short)12);
        // fontStyle.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        style.setFont(fontStyle);
        //设置边框
        style.setBorderTop(BorderStyle.THIN);
        style.setBorderRight(BorderStyle.THIN);
        style.setBorderBottom(BorderStyle.THIN);
        style.setBorderLeft(BorderStyle.THIN);


        // 新建sheet
        HSSFSheet sheet1 = wb.createSheet("Sheet1");

        int bIndex = basisInfo.size()-1;
        int fIndex = formInfo.isEmpty()?0:formInfo.size()-1;
        //设置第一行合并的单元格
        CellRangeAddress regionx1 = new CellRangeAddress(0, 0, 0, bIndex);
        CellRangeAddress regionx2 = new CellRangeAddress(0, 0, bIndex+1, fIndex+bIndex+1);
        sheet1.addMergedRegion(regionx1);
        sheet1.addMergedRegion(regionx2);

        //第一行标题
        HSSFRow row0 = sheet1.createRow(0);
//        HSSFCell cell_00 = row0.createCell(0);
//        cell_00.setCellValue("证照目录基本信息");
//        //cell_00.setCellStyle(style);
//        HSSFCell cell_01 = row0.createCell(bIndex+1);
//        cell_01.setCellValue("证照表单信息");
        //cell_01.setCellStyle(style);

        // 第二行 表头设置
        HSSFRow row1 = sheet1.createRow(1);
        List<String> cellList = new ArrayList<>();
        cellList.addAll(basisInfo);
        cellList.addAll(formInfo);
        for (int i = 0; i < cellList.size(); i++) {
            String s = cellList.get(i);
            HSSFCell cell_0 = row0.createCell(i);
            if(i==0){
                cell_0.setCellValue("证照目录基本信息(颁发时间格式:yyyy-mm-dd)");
            }
            if(i==(bIndex+1)){
                cell_0.setCellValue("证照表单信息");
            }
            cell_0.setCellStyle(style);
            HSSFCell cell = row1.createCell(i);
            // 设置每列的列宽
            sheet1.setColumnWidth(i, 4000);
            //sheet1.autoSizeColumn(i);
            //sheet1.setColumnWidth(i,sheet1.getColumnWidth(i)*17/10);
            //加样式
            cell.setCellStyle(style);
            cell.setCellValue(s);

            //CellRangeAddressList range = new CellRangeAddressList(2,65535,0,0);
            //DVConstraint constraint = DVConstraint.createTimeConstraint()

        }

        String[] str1 = {"自然人","法人","自然人、法人"};
        String[] str2 = {"身份证","组织机构代码"};
        // 255以内的下拉,参数分别是：作用的sheet、下拉内容数组、起始行、终止行、起始列、终止列
        sheet1.addValidationData(setDataValidation(sheet1, str1, 2, 500, 3 ,3)); //超过255个报错
        sheet1.addValidationData(setDataValidation(sheet1, str2, 2, 500, 4 ,4));
        CellStyle textStyle = wb.createCellStyle();
        DataFormat format = wb.createDataFormat();
        textStyle.setDataFormat(format.getFormat("@"));
        //证件号码，颁发时间设置为文本格式
        sheet1.setDefaultColumnStyle(5, textStyle);
        sheet1.setDefaultColumnStyle(7, textStyle);
        sheet1.setDefaultColumnStyle(8, textStyle);
        //所有表单设置为文本格式
        for(int cl=11;cl<formInfo.size();cl++){
            sheet1.setDefaultColumnStyle(cl, textStyle);
        }
        try {
            File f = new File(filePath); //写文件
            //不存在则新增
            if(!f.getParentFile().exists()){
                f.getParentFile().mkdirs();
            }
            if(!f.exists()){
                f.createNewFile();
            }
            FileOutputStream out = new FileOutputStream(f);
            out.flush();
            wb.write(out);
            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
