package com.mortals.xhx.common.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.xhx.common.code.HolderIdType;
import com.mortals.xhx.common.code.HolderType;
import com.mortals.xhx.module.record.model.ApplyLogEntity;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ooxml.POIXMLDocumentPart;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.*;
import org.openxmlformats.schemas.drawingml.x2006.spreadsheetDrawing.CTMarker;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
public class ImportExcelUtil {

    private static final String EXCEL_XLS_SUFFIX = ".xls";

    private static final String EXCEL_XLSX_SUFFIX = ".xlsx";

    /**
     * 读取Excel数据内容
     *
     * @param rowIndex      指定行号
     * @param columnIndex 指定列号
     * @return Map 包含单元格数据内容的Map对象
     */
    public static List<Map<Integer, Object>> readExcelContent(String filepath,Integer startRow ,Integer rowIndex, Integer columnIndex) throws Exception {
        List<Map<Integer, Object>> returnList = new LinkedList<>();
        Workbook wb = null;
        Sheet sheet;
        Row row;
        try {
            InputStream is = new FileInputStream(filepath);
            if (filepath.endsWith(EXCEL_XLS_SUFFIX)) {
                wb = new HSSFWorkbook(is);
            } else if (filepath.endsWith(EXCEL_XLSX_SUFFIX)) {
                wb = new XSSFWorkbook(is);
            }
            if (wb == null) {
                throw new Exception("Workbook对象为空！");
            }
            sheet = wb.getSheetAt(0);
            //解析文件总行数、总列数
            int rowNum = rowIndex != null ? rowIndex : sheet.getLastRowNum();
            row = sheet.getRow(startRow);
            int colNum = columnIndex != null ? columnIndex : row.getLastCellNum();
            //循环列
            for (int colIndex = colNum; colIndex > 0; colIndex--) {
                Cell cell = row.getCell(colIndex);
                if (cell != null && !"".equals(cell.toString())) {
                    colNum = colIndex;
                    break;
                }
            }
            log.info("have data col:{}", colNum);
            // 正文内容应该从第二行开始,第一行为表头的标题
            for (int i = startRow; i <= rowNum; i++) {
                row = sheet.getRow(i);
                int j = 0;
                int size = (int) (colNum / .75f) + 1;
                //存储单元格数据
                Map<Integer, Object> cellValue = new LinkedHashMap<>(size);
                if (row == null) {
                    continue;
                }
                while (j <= colNum) {
                    Cell cell = row.getCell(j);
                    System.out.println("j:"+j+",cell:"+cell.toString());
                    if(j==8){
                        System.out.println(cell.toString());
                    }
                    String value = "";
                    //日期单元格需格式化日期
                    if (cell != null) {
                        if (cell.getCellType() == CellType.NUMERIC) {
                            if (DateUtil.isCellDateFormatted(cell)) {
                                Date d = cell.getDateCellValue();
                                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
                                value = formatter.format(d);
                            } else if (cell.toString().contains("E")) {
                                DecimalFormat nf = new DecimalFormat("0");
                                value = nf.format(cell.getNumericCellValue());
                            } else {
                                value = cell.toString().endsWith(".0") ? cell.toString().replace(".0", "") : cell.toString().trim();
                            }
                        } else if (cell.getCellType() == CellType.FORMULA) {
                            value = String.valueOf(cell.getNumericCellValue());
                        } else {
                            value = cell.toString().trim();
                        }
                    }
                    cellValue.put(j, value);
                    j++;
                }
                returnList.add(cellValue);
            }
            wb.close();
        } catch (FileNotFoundException e) {
            log.error("FileNotFoundException", e);
        } catch (IOException e) {
            log.error("IOException", e);
        } finally {
            if (wb != null) {
                wb.close();
            }
        }
        return returnList;
    }

    /**
     * 读取Excel数据内容
     * @param startRow 从第几行开始读取
     * @param rowIndex      指定行号
     * @param columnIndex 指定列号
     * @return Map 包含单元格数据内容的Map对象
     */
    public static List<Map<Integer, Object>> readExcelContent(String fileName,InputStream is,Integer startRow, Integer rowIndex, Integer columnIndex) throws Exception {
        List<Map<Integer, Object>> returnList = new LinkedList<>();
        Workbook wb = null;
        Sheet sheet;
        Row row;
        try {
            if (fileName.endsWith(EXCEL_XLS_SUFFIX)) {
                wb = new HSSFWorkbook(is);
            } else if (fileName.endsWith(EXCEL_XLSX_SUFFIX)) {
                wb = new XSSFWorkbook(is);
            }
            if (wb == null) {
                throw new Exception("Workbook对象为空！");
            }
            sheet = wb.getSheetAt(0);
            //解析文件总行数、总列数
            int rowNum = rowIndex != null ? rowIndex : sheet.getLastRowNum();
            row = sheet.getRow(startRow);
            int colNum = columnIndex != null ? columnIndex : row.getLastCellNum();
            //循环列
            for (int colIndex = colNum; colIndex > 0; colIndex--) {
                Cell cell = row.getCell(colIndex);
                if (cell != null && !"".equals(cell.toString())) {
                    colNum = colIndex;
                    break;
                }
            }
            log.info("have data col:{}", colNum);

            // 正文内容应该从第二行开始,第一行为表头的标题
            for (int i = startRow; i <= rowNum; i++) {
                row = sheet.getRow(i);
                int j = 0;
                int size = (int) (colNum / .75f) + 1;
                //存储单元格数据
                Map<Integer, Object> cellValue = new LinkedHashMap<>(size);
                if (row == null) {
                    continue;
                }
                while (j <= colNum) {
                    Cell cell = row.getCell(j);
                    String value = "";
                    //日期单元格需格式化日期
                    if (cell != null) {
                        if (cell.getCellType() == CellType.NUMERIC) {
                            if (DateUtil.isCellDateFormatted(cell)) {
                                Date d = cell.getDateCellValue();
                                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
                                value = formatter.format(d);
                            } else if (cell.toString().contains("E")) {
                                DecimalFormat nf = new DecimalFormat("0");
                                value = nf.format(cell.getNumericCellValue());
                            } else {
                                value = cell.toString().endsWith(".0") ? cell.toString().replace(".0", "") : cell.toString().trim();
                            }
                        } else if (cell.getCellType() == CellType.FORMULA) {
                            value = String.valueOf(cell.getNumericCellValue());
                        } else {
                            value = cell.toString().trim();
                        }
                    }
                    cellValue.put(j, value);
                    j++;
                }
                returnList.add(cellValue);
            }
            wb.close();
        } catch (FileNotFoundException e) {
            log.error("FileNotFoundException", e);
        } catch (IOException e) {
            log.error("IOException", e);
        } finally {
            if (wb != null) {
                wb.close();
            }
        }
        return returnList;
    }

    public static Map<String, PictureData> getPictures(HSSFSheet sheet) {
        Map<String, PictureData> map = new HashMap();
        if (sheet.getDrawingPatriarch() == null) {
            return null;
        } else {
            List<HSSFShape> list = sheet.getDrawingPatriarch().getChildren();
            Iterator var4 = list.iterator();

            while(var4.hasNext()) {
                HSSFShape shape = (HSSFShape)var4.next();
                if (shape instanceof HSSFPicture) {
                    HSSFPicture picture = (HSSFPicture)shape;
                    HSSFClientAnchor cAnchor = (HSSFClientAnchor)picture.getAnchor();
                    HSSFPictureData pdata = picture.getPictureData();
                    String key = cAnchor.getRow1() + "-" + cAnchor.getCol1();
                    map.put(key, pdata);
                }
            }

            return map;
        }
    }

    public static Map<String, PictureData> getPictures(XSSFSheet sheet) {
        Map<String, PictureData> map = new HashMap();
        List<POIXMLDocumentPart> list = sheet.getRelations();
        Iterator var4 = list.iterator();

        while(true) {
            POIXMLDocumentPart part;
            do {
                if (!var4.hasNext()) {
                    return map;
                }

                part = (POIXMLDocumentPart)var4.next();
            } while(!(part instanceof XSSFDrawing));

            XSSFDrawing drawing = (XSSFDrawing)part;
            List<XSSFShape> shapes = drawing.getShapes();
            Iterator var8 = shapes.iterator();

            while(var8.hasNext()) {
                XSSFShape shape = (XSSFShape)var8.next();
                XSSFPicture picture = (XSSFPicture)shape;
                XSSFClientAnchor anchor = picture.getPreferredSize();
                CTMarker marker = anchor.getFrom();
                String key = marker.getRow() + "-" + marker.getCol();
                map.put(key, picture.getPictureData());
            }
        }
    }

    public static void main(String[] args){
        String filepath = "D:\\mortals\\app\\data\\cpm\\excel\\test3.xls";
        try {
            List<Map<Integer, Object>> dataList = ImportExcelUtil.readExcelContent(filepath,1,null,null);
            dataList.forEach(m->{
                System.out.println(m);
            });
            Map<Integer, Object> header = dataList.get(0);
            List<ApplyLogEntity> applyLogList = new ArrayList<>();
            String cf="{\"config\":{\"disabled\":false,\"dynamicTableAllowed\":true,\"formModel\":\"form\",\"formRef\":\"elForm\",\"formRules\":\"rules\",\"gutter\":15,\"labelPosition\":\"top\",\"labelWidth\":120,\"rules\":\"rules\",\"size\":\"medium\",\"version\":\"1.10\"},\"list\":[{\"_id\":\"i_1_姓名\",\"append\":\"\",\"clearable\":true,\"compIcon\":\"input\",\"compName\":\"证书名称\",\"compType\":\"input\",\"config\":true,\"disabled\":false,\"ele\":\"el-input\",\"gutter\":15,\"id\":\"i_2_证书名称\",\"label\":\"证书名称\",\"labelWidth\":120,\"layout\":\"colItem\",\"maxLength\":50,\"placeholder\":\"请输入证书名称\",\"prefix-icon\":\"\",\"prepend\":\"\",\"readonly\":false,\"required\":true,\"rules\":[],\"rulesType\":\"default\",\"showLabel\":true,\"span\":24,\"status\":\"normal\",\"viewType\":\"text\",\"width\":\"100%\"},{\"_id\":\"i_2_性别\",\"append\":\"\",\"clearable\":true,\"compIcon\":\"input\",\"compName\":\"证书编号\",\"compType\":\"input\",\"config\":true,\"disabled\":false,\"ele\":\"el-input\",\"gutter\":15,\"id\":\"i_1_证书编号\",\"label\":\"证书编号\",\"labelWidth\":120,\"layout\":\"colItem\",\"maxLength\":50,\"placeholder\":\"请输入证书编号\",\"prefix-icon\":\"\",\"prepend\":\"\",\"readonly\":false,\"required\":true,\"rules\":[],\"rulesType\":\"default\",\"showLabel\":true,\"span\":24,\"status\":\"normal\",\"viewType\":\"text\",\"width\":\"100%\"}]}";
            JSONObject formContent = JSONObject.parseObject(cf);
            JSONArray formList = formContent.getJSONArray("list");
            for (int r=1;r<dataList.size();r++){
                Map<Integer, Object> row = dataList.get(r);
                ApplyLogEntity entity = new ApplyLogEntity();

                entity.setCertificateCode(String.valueOf(row.get(0)));
                entity.setCertificateName(String.valueOf(row.get(1)));
                entity.setHolderName(String.valueOf(row.get(2)));
                HolderType holderType = HolderType.getByDesc(String.valueOf(row.get(3)));
                entity.setHolderType(holderType==null?HolderType.LEGAL_AND_PERSON.getValue():holderType.getValue());
                HolderIdType holderIdType = HolderIdType.getByDesc(String.valueOf(row.get(4)));
                entity.setHolderIdType(holderIdType==null?HolderIdType.ID_CARD.getValue():holderIdType.getValue());
                entity.setHolderIDCardNo(String.valueOf(row.get(5)));
                entity.setPickerName(String.valueOf(row.get(6)));
                entity.setPickerIDCardNo(String.valueOf(row.get(7)));

                try {
                    SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
                    entity.setIssueTime(formatter.parse(String.valueOf(row.get(8))));
                }catch (ParseException e){
                    entity.setIssueTime(new Date());
                }
                entity.setPrivateID(String.valueOf(row.get(9)));
                entity.setEnterpriseName(String.valueOf(row.get(10)));
                JSONObject formJson = new JSONObject();
                int formSize = row.size();
                for(int i = 11;i<formSize;i++) {
                    formJson.put(String.valueOf(header.get(i)), String.valueOf(row.get(i)));

                    for (int j = 0; j < formList.size(); j++) {
                        JSONObject jsonObject = formList.getJSONObject(j);
                        if(jsonObject.getString("_id").equals(String.valueOf(header.get(i)))){
                            jsonObject.put("value",String.valueOf(row.get(i)));
                            break;
                        }
                    }
                }
                System.out.println(formJson.toJSONString());
                entity.setFormContent(formJson.toJSONString());
                System.out.println(formContent.toJSONString());
                entity.setFormTemplate(formContent.toJSONString());

                entity.setCreateTime(new Date());
                System.out.println(entity.toString());
            }
        }catch (Exception e){
            e.printStackTrace();
        }

    }
}
